###########################################################################
##                                                                       ##
##  Replication script for "Burden Sharing in CSDP Military Operations"  ##
##                                                                       ##
##  Authors: Tim Haesebrouck, Alrik Thiem                                ##
##                                                                       ##
##  Date   : 27 April 2017                                               ##
##                                                                       ##
###########################################################################

# load packages
###############
library(ppcor)
library(QCApro)


####################################
# Analysis Distribution of Burdens #
####################################

# load data
dat <- read.csv("base_contributor_status.csv", row.names = "case")
head(dat)

# formula for burden index
burden <- function(OM, TN, UT, LR, a.1, a.2, a.3){
 (TN^(a.1) * UT^(a.2) * LR^(a.3) /
  sum(TN^(a.1) * UT^(a.2) * LR^(a.3)))
}


# create burden index variable and compute ranks
################################################

# set-up
dat$burd.ind <- numeric(nrow(dat))

# ranks, EUFOR Congo
dat$burd.ind[1:25] <- burden(dat$dura[1:25], dat$trp[1:25],
                             dat$unit[1:25], dat$lead[1:25],
                             0.6, 0.3, 0.1)

cbind(dat[1:25, ][order(dat[1:25, "burd.ind"], decreasing = TRUE), "burd.ind", 
 drop = FALSE], rank = c(1:9, NA, 10:13, rep(c(16, 24), c(3, 8))))

# ranks, EUFOR Chad
dat$burd.ind[26:54] <- burden(dat$dura[26:54], dat$trp[26:54], 
                              dat$unit[26:54], dat$lead[26:54],
                              0.6, 0.3, 0.1)

cbind(dat[26:54, ][order(dat[26:54, "burd.ind"], decreasing = TRUE), "burd.ind", 
 drop = FALSE], 
 rank = c(1:9, NA, 10, NA, NA, 11, 12, rep(14, 2), 15, 16, rep(22, 6), 23, rep(26, 3)))
                              
# ranks, both EUFORs
comb <- data.frame(matrix(numeric(24), ncol = 1, 
 dimnames = list(
  unlist(strsplit(rownames(dat)[c(27,28,30:36,38:47,50:54)], "_"))[seq(1,48,2)], 
  "burd.com"))
)

# purge non-EU countries
(dat.c <- dat[-c(24, 26, 29, 37, 48, 49), ])

# calculate burden index
for(i in 1:24){
 comb[i, 1] <- sum( 
  prod(sort(unique(dat$dura))[1]/sum(unique(dat$dura)), dat.c$burd.ind[i]), 
  prod(sort(unique(dat$dura))[2]/sum(unique(dat$dura)), dat.c$burd.ind[24 + i])
  )  
}

# ranks, combined 
cbind(apply(comb, 2, sort, decreasing = TRUE), 
 rank = c(1:15, rep(17, 2), 18, rep(20, 2), 21, rep(24, 3)))

# purge non-EU countries
(dat.k <- dat[-c(24, 26, 37, 49), ])

# relative troop numbers, GDP, population, armed forces personnel
dat.k$gdp.sh <- c(dat.k$gdp[1:24 ]/sum(dat.k$gdp[1:24]), 
                dat.k$gdp[25:50]/sum(dat.k$gdp[25:50]))

dat.k$pop.sh <- c(dat.k$pop[1:24 ]/sum(dat.k$pop[1:24]),
                dat.k$pop[25:50]/sum(dat.k$pop[25:50]))

dat.k$mil.sh <- c(dat.k$mil[1:24 ]/sum(dat.k$mil[1:24]), 
                dat.k$mil[25:50]/sum(dat.k$mil[25:50]))

dat.c$gdp.sh <- dat.k$gdp.sh[-c(27, 45)] 
dat.c$pop.sh <- dat.k$pop.sh[-c(27, 45)]
dat.c$mil.sh <- dat.k$mil.sh[-c(27, 45)]

comb$gdp.sh <- comb$pop.sh <- comb$mil.sh <- numeric(24)

for(i in 1:24){
 comb$gdp.sh[i] <- (dat.c$gdp[i] + dat.c$gdp[24 + i]) / sum(dat.c$gdp)
 comb$pop.sh[i] <- (dat.c$pop[i] + dat.c$pop[24 + i]) / sum(dat.c$pop)
 comb$mil.sh[i] <- (dat.c$mil[i] + dat.c$mil[24 + i]) / sum(dat.c$mil)
} 


# simple Kendall tests
######################

# GDP
pcor(cbind(dat.k$burd.ind[1:24] / dat.k$gdp.sh[1:24], dat.k$gdp.sh[1:24]),
  method = "kendall")[1:3] # Congo

pcor(cbind(dat.k$burd.ind[25:50] / dat.k$gdp.sh[25:50], dat.k$gdp.sh[25:50]),
  method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd/comb$gdp.sh, comb$gdp.sh), method = "kendall")[1:3] # combined

# population
pcor(cbind(dat.k$burd.ind[1:24] / dat.k$pop.sh[1:24], dat.k$pop.sh[1:24]), 
  method = "kendall")[1:3]# Congo

pcor(cbind(dat.k$burd.ind[25:50]/dat.k$pop.sh[25:50], dat.k$pop.sh[25:50]),
  method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd/comb$pop.sh, comb$pop.sh), method = "kendall")[1:3] # combined

# GDP and population
pcor(cbind(dat.k$burd.ind[1:24]/((dat.k$pop.sh[1:24] + dat.k$gdp.sh[1:24])/2), 
  (dat.k$pop.sh[1:24] + dat.k$gdp.sh[1:24])/2), method = "kendall")[1:3] # Congo

pcor(cbind(dat.k$burd.ind[25:50]/((dat.k$pop.sh[25:50] + dat.k$gdp.sh[25:50])/2),
  (dat.k$pop.sh[25:50] + dat.k$gdp.sh[25:50])/2), method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd /((comb$pop.sh + comb$gdp.sh)/2), (comb$pop.sh + comb$gdp.sh)/2),
  method = "kendall")[1:3] # combined


# partial Kendall tests
#######################

# GDP
pcor(cbind(dat.k$burd.ind[1:24] / dat.k$gdp.sh[1:24], dat.k$gdp.sh[1:24],
  dat.k$mil.sh[1:24]), method = "kendall")[1:3] # Congo

pcor(cbind(dat.k$burd.ind[25:50]/dat.k$gdp.sh[25:50], dat.k$gdp.sh[25:50],
  dat.k$mil.sh[25:50]), method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd/comb$gdp.sh, comb$gdp.sh, comb$mil.sh),
  method = "kendall")[1:3] # combined

# population
pcor(cbind(dat.k$burd.ind[1:24] / dat.k$pop.sh[1:24], dat.k$pop.sh[1:24], 
  dat.k$mil.sh[1:24]), method = "kendall")[1:3] # Congo

pcor(cbind(dat.k$burd.ind[25:50]/dat.k$pop.sh[25:50], dat.k$pop.sh[25:50],
  dat.k$mil.sh[25:50]), method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd/comb$pop.sh, comb$pop.sh, comb$mil.sh),
  method = "kendall")[1:3] # combined

# GDP and population
pcor(cbind(dat.k$burd.ind[1:24] / ((dat.k$pop.sh[1:24] + dat.k$gdp.sh[1:24])/2), 
  (dat.k$pop.sh[1:24] + dat.k$gdp.sh[1:24])/2, dat.k$mil.sh[1:24]), 
  method = "kendall")[1:3] # Congo

pcor(cbind(dat.k$burd.ind[25:50]/((dat.k$pop.sh[25:50] + dat.k$gdp.sh[25:50])/2),
  (dat.k$pop.sh[25:50] + dat.k$gdp.sh[25:50])/2, dat.k$mil[25:50]), 
  method = "kendall")[1:3] # Chad

pcor(cbind(comb$burd/((comb$pop.sh + comb$gdp.sh)/2), (comb$pop.sh + comb$gdp.sh)/2,
  comb$mil.sh), method = "kendall")[1:3] # combined

# expected burden shares
########################

# Congo
shares.Congo <- matrix(
 c(dat.k$gdp.sh[1:24], dat.k$pop.sh[1:24],
  (dat.k$pop.sh[1:24] + dat.k$gdp.sh[1:24])/2, dat.k$burd.ind[1:24]),
   byrow = FALSE, ncol = 4, dimnames = list(
    rownames(comb), c("gdp", "pop", "gdppop", "obs")
   )
)

# Chad
shares.Chad <- matrix(
 c(dat.k$gdp.sh[25:50], dat.k$pop.sh[25:50],
  (dat.k$pop.sh[25:50] + dat.k$gdp.sh[25:50])/2, dat.k$burd.ind[25:50]),
   byrow = FALSE, ncol = 4, dimnames = list(
    unlist(strsplit(rownames(dat.k)[c(25:50)], "_"))[seq(1,52,2)],
    c("gdp", "pop", "gdppop", "obs")
   )
)

# Combined
shares.Comb <- matrix(
 c(comb$gdp.sh, comb$pop.sh, (comb$pop.sh + comb$gdp.sh)/2, comb$burd.com),
   byrow = FALSE, ncol = 4, dimnames = list(
    rownames(comb), c("gdp", "pop", "gdppop", "obs")
   )
)


# plot observed burden shares against expected shares based on GDP,
# population and relative averaged combination of GDP and population
####################################################################

title.add <- c("GDP", "Population", "GDP and Pop.")
par(mfrow = c(3, 3), mar = c(4, 4, 3, 1) + 0.1)
for(i in 1:3){
plot(shares.Congo[ , i], shares.Congo[ ,"obs"], main = paste("Congo (", title.add[i], ")",
 sep = ""), xlab = "Expected Burden Share", ylab = "Observed Burden Share",
 axes = FALSE, type = "n", xlim = c(0,0.35), ylim = c(0,0.35))
axis(1, at = c(0,0.1,0.2,0.3,0.35))
axis(2, at = c(0,0.1,0.2,0.3,0.35), las = 2)
segments(-0.02, -0.02, 0.29, 0.29)
abline(coef = c(0,1.5), lty = 2)
abline(coef = c(0,2/3), lty = 2)
points(shares.Congo[ , i], shares.Congo[ ,"obs"], pch = 21, cex = 1.2,
 bg = ifelse(shares.Congo[, "obs"] > 1.5*shares.Congo[ , i], grey(0.95),
       ifelse(shares.Congo[, "obs"] < (2/3)*shares.Congo[ , i], grey(0), grey(0.48))))
text(c(0.325,0.325,0.03), c(0.03,0.33,0.33), labels = c(expression(italic(CS)^{paste("{U}")}),
  expression(italic(CS)^{paste("{E}")}), expression(italic(CS)^{paste("{O}")})))
identify(shares.Congo[ , i], shares.Congo[ , "obs"], labels = rownames(shares.Congo),
 offset = 0.75)
}

for(i in 1:3){
plot(shares.Chad[ , i], shares.Chad[ ,"obs"], main = paste("Chad (", title.add[i], ")",
 sep = ""), xlab = "Expected Burden Share", ylab = "Observed Burden Share",
 axes = FALSE, type = "n", xlim = c(0,0.35), ylim = c(0,0.35))
axis(1, at = c(0,0.1,0.2,0.3,0.35))
axis(2, at = c(0,0.1,0.2,0.3,0.35), las = 2)
segments(-0.02, -0.02, 0.29, 0.29)
abline(coef = c(0,1.5), lty = 2)
abline(coef = c(0,2/3), lty = 2)
points(shares.Chad[ , i], shares.Chad[ ,"obs"], pch = 21, cex = 1.2,
 bg = ifelse(shares.Chad[, "obs"] > 1.5*shares.Chad[ , i], grey(0.95),
       ifelse(shares.Chad[, "obs"] < (2/3)*shares.Chad[ , i], grey(0), grey(0.48))))
text(c(0.325,0.325,0.03), c(0.03,0.33,0.33), labels = c(expression(italic(CS)^{paste("{U}")}),
  expression(italic(CS)^{paste("{E}")}), expression(italic(CS)^{paste("{O}")})))
identify(shares.Chad[ , i], shares.Chad[ , "obs"], labels = rownames(shares.Chad),
 offset = 0.75)
}

for(i in 1:3){
plot(shares.Comb[ , i], shares.Comb[ ,"obs"], main = paste("Comb. (", title.add[i], ")",
 sep = ""), xlab = "Expected Burden Share", ylab = "Observed Burden Share",
 axes = FALSE, type = "n", xlim = c(0,0.35), ylim = c(0,0.35))
axis(1, at = c(0,0.1,0.2,0.3,0.35))
axis(2, at = c(0,0.1,0.2,0.3,0.35), las = 2)
segments(-0.02, -0.02, 0.29, 0.29)
abline(coef = c(0,1.5), lty = 2)
abline(coef = c(0,2/3), lty = 2)
points(shares.Comb[ , i], shares.Comb[ ,"obs"], pch = 21, cex = 1.2,
 bg = ifelse(shares.Comb[, "obs"] > 1.5*shares.Comb[ , i], grey(0.95),
       ifelse(shares.Comb[, "obs"] < (2/3)*shares.Comb[ , i], grey(0), grey(0.48))))
text(c(0.325,0.325,0.03), c(0.03,0.33,0.33), labels = c(expression(italic(CS)^{paste("{U}")}),
  expression(italic(CS)^{paste("{E}")}), expression(italic(CS)^{paste("{O}")})))
identify(shares.Comb[ , i], shares.Comb[ , "obs"], labels = rownames(shares.Comb),
 offset = 0.75)
}


######################
# Analysis QCA Model #
######################

# load base variables
base <- read.csv("base_all.csv", row.names = "case")
head(base)

# summary statistics
apply(base, 2, summary, details = TRUE)

# calibration; first two letters are factor; last letter is factor level;
# for endogenous factor, last three letters are burden indicator
#########################################################################

dat <- data.frame(matrix(
 numeric(50*12), nrow = 50,
  dimnames = list(row.names(base),
   c("PTS","CDS","TVL","PSH","BCH","EDL","PPS","EPR","LPR",
     "CSGDP","CSPOP","CSCOM")
  )
 )
)

# calibrate exogenous factors (crisp sets)
# positive public benefits
dat$PTS <- calibrate(base$ppi, thresholds = 1)    # peacekeeping tradition, strong
# negative public benefits
dat$CDS <- calibrate(base$mcs, thresholds = 15)   # competing deployments, significant
# positive private benefits
dat$TVL <- calibrate(base$tvr, thresholds = 0.5)  # trade volume, large
dat$PSH <- calibrate(base$nps, thresholds = 30)   # public support, high
# negative private benefits
dat$BCH <- calibrate(base$bcs, thresholds = 0.5)  # budget constraints, high
dat$EDL <- calibrate(base$tre, thresholds = 365)  # election distance, large
dat$PPS <- calibrate(base$ppg, thresholds = 0.5)  # parliamentary powers, significant
# positive or negative private benefits
dat$EPR <- calibrate(base$eio, thresholds = 5)    # executive partisanship, right
dat$LPR <- calibrate(base$pio, thresholds = 5.25) # legislative partisanship, right

head(dat)

# calibrate endogenous factor for burden share ratios
#####################################################

# compute ratios between observed and expected burden shares
r.gdp <- r.pop <- r.gdppop <- numeric(nrow(shares.Congo))
shares.Congo <- cbind(shares.Congo, r.gdp, r.pop, r.gdppop)
shares.Comb <- cbind(shares.Comb, r.gdp, r.pop, r.gdppop)
r.gdp <- r.pop <- r.gdppop <- numeric(nrow(shares.Chad))
shares.Chad <- cbind(shares.Chad, r.gdp, r.pop, r.gdppop)

for(i in 5:7){
 shares.Congo[,i] <- shares.Congo[, "obs"] / shares.Congo[, i-4]
 shares.Chad[,i] <- shares.Chad[, "obs"] / shares.Chad[, i-4]
 shares.Comb[,i] <- shares.Comb[, "obs"] / shares.Comb[, i-4]
}

(shares.Congo <- as.data.frame(round(shares.Congo, 4)))
(shares.Chad <- as.data.frame(round(shares.Chad, 4)))
(shares.Comb <- as.data.frame(round(shares.Comb, 4)))

# GDP
dat$CSGDP <- calibrate(c(shares.Congo$r.gdp, shares.Chad$r.gdp),
 thresholds = c(2/3,1.5))

# population
dat$CSPOP <- calibrate(c(shares.Congo$r.pop, shares.Chad$r.pop),
 thresholds = c(2/3,1.5))

# GDP and population
dat$CSCOM <- calibrate(c(shares.Congo$r.gdppop, shares.Chad$r.gdppop),
 thresholds = c(2/3,1.5))


# set up factor combinations and split data frames
##################################################

# all 105 five-factor combinations that include at least one factor
# from public good theory to retain joint product model
cbn <- t(combn(names(dat)[1:9], 5))
cbn <- cbn[cbn[, 1] == "PTS" | cbn[, 1] == "CDS", ]

# EUFOR Congo
dat.CO <- dat[1:24,]
# EUFOR Chad
dat.CH <- dat[25:50,]


# find best-fitting factor combinations
#######################################

# GDP
#####
N.mods.gdp <- matrix(numeric(9*nrow(cbn)), ncol = 9,
 dimnames = list(1:nrow(cbn),
 c("n.mod.u", "n.pi.u", "ic.max.u",
   "n.mod.e", "n.pi.e", "ic.max.e",
   "n.mod.o", "n.pi.o", "ic.max.o")))

for(i in seq(nrow(cbn))){
 tryCatch({
 sol.u <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSGDP{0}",
  incl.cut1 = 0.75)
  N.mods.gdp[i,1] <- length(sol.u$solution)
  N.mods.gdp[i,2] <- length(sol.u$solution[[1]])
  if (N.mods.gdp[i, 1] > 1) {
      n <- N.mods.gdp[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdp[i, 3] <- round(max(p), 3)
  }
  else {
      N.mods.gdp[i, 3] <- round(sol.u$IC$sol.incl.cov[1] * sol.u$IC$sol.incl.cov[3], 3)
  }
 sol.e <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSGDP{1}",
  incl.cut1 = 0.75)
  N.mods.gdp[i,4] <- length(sol.e$solution)
  N.mods.gdp[i,5] <- length(sol.e$solution[[1]])
  if (N.mods.gdp[i, 4] > 1) {
      n <- N.mods.gdp[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdp[i, 6] <- round(max(p), 3)
  }
  else {
      N.mods.gdp[i, 6] <- round(sol.e$IC$sol.incl.cov[1] * sol.e$IC$sol.incl.cov[3], 3)
  }
 sol.o <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSGDP{2}",
  incl.cut1 = 0.75)
  N.mods.gdp[i,7] <- length(sol.o$solution)
  N.mods.gdp[i,8] <- length(sol.o$solution[[1]])
  if (N.mods.gdp[i, 7] > 1) {
      n <- N.mods.gdp[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdp[i, 9] <- round(max(p), 3)
  }
  else {
      N.mods.gdp[i, 9] <- round(sol.o$IC$sol.incl.cov[1] * sol.o$IC$sol.incl.cov[3], 3)
  }
 }, error = function(e) NULL)
}

# underproviders
N.mods.gdp[N.mods.gdp[ , 1] <= 1 &
           N.mods.gdp[ , 2] >= 2 &
           N.mods.gdp[ , 3] >= 0.5, 1:3, drop = FALSE]

# equiproviders
N.mods.gdp[N.mods.gdp[ , 4] <= 1 &
           N.mods.gdp[ , 5] == 2 &
           N.mods.gdp[ , 6] >= 0.5, 4:6, drop = FALSE]

# overproviders
N.mods.gdp[N.mods.gdp[ , 7] <= 1 &
           N.mods.gdp[ , 8] == 2 &
           N.mods.gdp[ , 9] >= 0.5, 7:9, drop = FALSE]

eQMC(dat, outcome = "CSGDP{0}", exo.facs = cbn[5,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSGDP{1}", exo.facs = cbn[3,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSGDP{2}", exo.facs = cbn[36,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)


# population
############
N.mods.pop <- matrix(numeric(9*nrow(cbn)), ncol = 9,
 dimnames = list(1:nrow(cbn),
 c("n.mod.u", "n.pi.u", "ic.max.u",
   "n.mod.e", "n.pi.e", "ic.max.e",
   "n.mod.o", "n.pi.o", "ic.max.o")))

for(i in seq(nrow(cbn))){
 tryCatch({
 sol.u <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSPOP{0}",
  incl.cut1 = 0.75)
  N.mods.pop[i,1] <- length(sol.u$solution)
  N.mods.pop[i,2] <- length(sol.u$solution[[1]])
  if (N.mods.pop[i, 1] > 1) {
      n <- N.mods.pop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.pop[i, 3] <- round(max(p), 3)
  }
  else {
      N.mods.pop[i, 3] <- round(sol.u$IC$sol.incl.cov[1] * sol.u$IC$sol.incl.cov[3], 3)
  }
 sol.e <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSPOP{1}",
  incl.cut1 = 0.75)
  N.mods.pop[i,4] <- length(sol.e$solution)
  N.mods.pop[i,5] <- length(sol.e$solution[[1]])
  if (N.mods.pop[i, 4] > 1) {
      n <- N.mods.pop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.pop[i, 6] <- round(max(p), 3)
  }
  else {
      N.mods.pop[i, 6] <- round(sol.e$IC$sol.incl.cov[1] * sol.e$IC$sol.incl.cov[3], 3)
  }
 sol.o <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSPOP{2}",
  incl.cut1 = 0.75)
  N.mods.pop[i,7] <- length(sol.o$solution)
  N.mods.pop[i,8] <- length(sol.o$solution[[1]])
  if (N.mods.pop[i, 7] > 1) {
      n <- N.mods.pop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.pop[i, 9] <- round(max(p), 3)
  }
  else {
      N.mods.pop[i, 9] <- round(sol.o$IC$sol.incl.cov[1] * sol.o$IC$sol.incl.cov[3], 3)
  }
 }, error = function(e) NULL)
}

# underproviders
N.mods.pop[N.mods.pop[ , 1] <= 1 &
           N.mods.pop[ , 2] >= 2 &
           N.mods.pop[ , 3] >= 0.5, 1:3, drop = FALSE]

# equiproviders
N.mods.pop[N.mods.pop[ , 4] <= 1 &
           N.mods.pop[ , 5] == 2 &
           N.mods.pop[ , 6] >= 0.5, 4:6, drop = FALSE]

# overproviders
N.mods.pop[N.mods.pop[ , 7] <= 1 &
           N.mods.pop[ , 8] == 2 &
           N.mods.pop[ , 9] >= 0.4, 7:9, drop = FALSE]

eQMC(dat, outcome = "CSPOP{0}", exo.facs = cbn[5,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSPOP{1}", exo.facs = cbn[10,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSPOP{2}", exo.facs = cbn[11,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)


# GDP and population
####################
N.mods.gdppop <- matrix(numeric(9*nrow(cbn)), ncol = 9,
 dimnames = list(1:nrow(cbn),
 c("n.mod.u", "n.pi.u", "ic.max.u",
   "n.mod.e", "n.pi.e", "ic.max.e",
   "n.mod.o", "n.pi.o", "ic.max.o")))

for(i in seq(nrow(cbn))){
 tryCatch({
 sol.u <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSCOM{0}",
  incl.cut1 = 0.75)
  N.mods.gdppop[i,1] <- length(sol.u$solution)
  N.mods.gdppop[i,2] <- length(sol.u$solution[[1]])
  if (N.mods.gdppop[i, 1] > 1) {
      n <- N.mods.gdppop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.u$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdppop[i, 3] <- round(max(p), 3)
  }
  else {
      N.mods.gdppop[i, 3] <- round(sol.u$IC$sol.incl.cov[1] * sol.u$IC$sol.incl.cov[3], 3)
  }
 sol.e <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSCOM{1}",
  incl.cut1 = 0.75)
  N.mods.gdppop[i,4] <- length(sol.e$solution)
  N.mods.gdppop[i,5] <- length(sol.e$solution[[1]])
  if (N.mods.gdppop[i, 4] > 1) {
      n <- N.mods.gdppop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.e$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdppop[i, 6] <- round(max(p), 3)
  }
  else {
      N.mods.gdppop[i, 6] <- round(sol.e$IC$sol.incl.cov[1] * sol.e$IC$sol.incl.cov[3], 3)
  }
 sol.o <- eQMC(dat, exo.facs = cbn[i,], outcome = "CSCOM{2}",
  incl.cut1 = 0.75)
  N.mods.gdppop[i,7] <- length(sol.o$solution)
  N.mods.gdppop[i,8] <- length(sol.o$solution[[1]])
  if (N.mods.gdppop[i, 7] > 1) {
      n <- N.mods.gdppop[i, 1]
      p <- sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[1]}) *
           sapply(seq(n), function (x) {
             sol.o$IC$individual[[x]]$sol.incl.cov[3]})
      N.mods.gdppop[i, 9] <- round(max(p), 3)
  }
  else {
      N.mods.gdppop[i, 9] <- round(sol.o$IC$sol.incl.cov[1] * sol.o$IC$sol.incl.cov[3], 3)
  }
 }, error = function(e) NULL)
}

# underproviders
N.mods.gdppop[N.mods.gdppop[ , 1] <= 1 &
              N.mods.gdppop[ , 2] >= 2 &
              N.mods.gdppop[ , 3] >= 0.5, 1:3, drop = FALSE]

# equiproviders
N.mods.gdppop[N.mods.gdppop[ , 4] <= 1 &
              N.mods.gdppop[ , 5] == 2 &
              N.mods.gdppop[ , 6] >= 0.5, 4:6, drop = FALSE]

# overproviders
N.mods.gdppop[N.mods.gdppop[ , 7] <= 1 &
              N.mods.gdppop[ , 8] == 2 &
              N.mods.gdppop[ , 9] >= 0.4, 7:9, drop = FALSE]

eQMC(dat, outcome = "CSCOM{0}", exo.facs = cbn[5,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSCOM{1}", exo.facs = cbn[27,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)

eQMC(dat, outcome = "CSCOM{2}", exo.facs = cbn[72,], incl.cut1 = 0.75,
  details = TRUE, show.cases = TRUE)